@@ -65,10 +65,10 @@ settings. Here is an example::
         }
     ]
 
-Each element in the root list represents a setting that the user can configure.
-Only the "type" key is mandatory: an instance of the associated class will be
-created and used for the setting - other keys are assigned to corresponding
-properties of that class.
+Each element in the root list represents a setting that the user can
+configure. Only the "type" key is mandatory: an instance of the associated
+class will be created and used for the setting - other keys are assigned to
+corresponding properties of that class.
 
     ============== =================================================
     Type           Associated class
@@ -79,11 +79,15 @@ properties of that class.
     options        :class:`SettingOptions`
     string         :class:`SettingString`
     path           :class:`SettingPath`
+    color          :class:`SettingColor`
     ============== =================================================
 
     .. versionadded:: 1.1.0
         Added :attr:`SettingPath` type
 
+    .. versionadded:: 2.1.0
+        Added :attr:`SettingColor` type
+
 In the JSON example above, the first element is of type "title". It will create
 a new instance of :class:`SettingTitle` and apply the rest of the key-value
 pairs to the properties of that class, i.e. "title": "Windows" sets the
@@ -168,6 +172,7 @@ __all__ = ('Settings', 'SettingsPanel', 'SettingItem', 'SettingString',
 
 import json
 import os
+import kivy.utils as utils
 from kivy.factory import Factory
 from kivy.metrics import dp
 from kivy.config import ConfigParser
@@ -178,6 +183,7 @@ from kivy.uix.boxlayout import BoxLayout
 from kivy.uix.tabbedpanel import TabbedPanelHeader
 from kivy.uix.button import Button
 from kivy.uix.filechooser import FileChooserListView
+from kivy.uix.colorpicker import ColorPicker
 from kivy.uix.scrollview import ScrollView
 from kivy.uix.floatlayout import FloatLayout
 from kivy.uix.gridlayout import GridLayout
@@ -213,8 +219,8 @@ class SettingItem(FloatLayout):
     title = StringProperty('<No title set>')
     '''Title of the setting, defaults to '<No title set>'.
 
-    :attr:`title` is a :class:`~kivy.properties.StringProperty` and defaults to
-    '<No title set>'.
+    :attr:`title` is a :class:`~kivy.properties.StringProperty` and defaults
+    to '<No title set>'.
     '''
 
     desc = StringProperty(None, allownone=True)
@@ -270,8 +276,8 @@ class SettingItem(FloatLayout):
     As soon as the content object is set, any further call to add_widget will
     call the content.add_widget. This is automatically set.
 
-    :attr:`content` is an :class:`~kivy.properties.ObjectProperty` and defaults
-    to None.
+    :attr:`content` is an :class:`~kivy.properties.ObjectProperty` and
+    defaults to None.
     '''
 
     selected_alpha = NumericProperty(0)
@@ -288,10 +294,10 @@ class SettingItem(FloatLayout):
         super(SettingItem, self).__init__(**kwargs)
         self.value = self.panel.get_value(self.section, self.key)
 
-    def add_widget(self, *largs):
+    def add_widget(self, *args, **kwargs):
         if self.content is None:
-            return super(SettingItem, self).add_widget(*largs)
-        return self.content.add_widget(*largs)
+            return super(SettingItem, self).add_widget(*args, **kwargs)
+        return self.content.add_widget(*args, **kwargs)
 
     def on_touch_down(self, touch):
         if not self.collide_point(*touch.pos):
@@ -324,9 +330,10 @@ class SettingItem(FloatLayout):
 
 
 class SettingBoolean(SettingItem):
-    '''Implementation of a boolean setting on top of a :class:`SettingItem`. It
-    is visualized with a :class:`~kivy.uix.switch.Switch` widget. By default,
-    0 and 1 are used for values: you can change them by setting :attr:`values`.
+    '''Implementation of a boolean setting on top of a :class:`SettingItem`.
+    It is visualized with a :class:`~kivy.uix.switch.Switch` widget.
+    By default, 0 and 1 are used for values: you can change them by setting
+    :attr:`values`.
     '''
 
     values = ListProperty(['0', '1'])
@@ -518,6 +525,68 @@ class SettingPath(SettingItem):
         popup.open()
 
 
+class SettingColor(SettingItem):
+    '''Implementation of a color setting on top of a :class:`SettingItem`.
+    It is visualized with a :class:`~kivy.uix.label.Label` widget and a
+    colored canvas rectangle that, when clicked, will open a
+    :class:`~kivy.uix.popup.Popup` with a
+    :class:`~kivy.uix.colorpicker.ColorPicker` so the user can choose a color.
+
+    .. versionadded:: 2.0.1
+    '''
+
+    popup = ObjectProperty(None, allownone=True)
+    '''(internal) Used to store the current popup when it's shown.
+
+    :attr:`popup` is an :class:`~kivy.properties.ObjectProperty` and defaults
+    to None.
+    '''
+
+    def on_panel(self, instance, value):
+        if value is None:
+            return
+        self.bind(on_release=self._create_popup)
+
+    def _dismiss(self, *largs):
+        if self.popup:
+            self.popup.dismiss()
+        self.popup = None
+
+    def _validate(self, instance):
+        self._dismiss()
+        value = utils.get_hex_from_color(self.colorpicker.color)
+        self.value = value
+
+    def _create_popup(self, instance):
+        # create popup layout
+        content = BoxLayout(orientation='vertical', spacing='5dp')
+        popup_width = min(0.95 * Window.width, dp(500))
+        self.popup = popup = Popup(
+            title=self.title, content=content, size_hint=(None, 0.9),
+            width=popup_width)
+
+        self.colorpicker = colorpicker = \
+            ColorPicker(color=utils.get_color_from_hex(self.value))
+        colorpicker.bind(on_color=self._validate)
+
+        self.colorpicker = colorpicker
+        content.add_widget(colorpicker)
+        content.add_widget(SettingSpacer())
+
+        # 2 buttons are created for accept or cancel the current value
+        btnlayout = BoxLayout(size_hint_y=None, height='50dp', spacing='5dp')
+        btn = Button(text='Ok')
+        btn.bind(on_release=self._validate)
+        btnlayout.add_widget(btn)
+        btn = Button(text='Cancel')
+        btn.bind(on_release=self._dismiss)
+        btnlayout.add_widget(btn)
+        content.add_widget(btnlayout)
+
+        # all done, open the popup !
+        popup.open()
+
+
 class SettingNumeric(SettingString):
     '''Implementation of a numeric setting on top of a :class:`SettingString`.
     It is visualized with a :class:`~kivy.uix.label.Label` widget that, when
@@ -609,7 +678,7 @@ class SettingTitle(Label):
 
 
 class SettingsPanel(GridLayout):
-    '''This class is used to contruct panel settings, for use with a
+    '''This class is used to construct panel settings, for use with a
     :class:`Settings` instance or subclass.
     '''
 
@@ -629,8 +698,7 @@ class SettingsPanel(GridLayout):
     '''
 
     def __init__(self, **kwargs):
-        if 'cols' not in kwargs:
-            self.cols = 1
+        kwargs.setdefault('cols', 1)
         super(SettingsPanel, self).__init__(**kwargs)
 
     def on_config(self, instance, value):
@@ -869,14 +937,14 @@ class ContentPanel(ScrollView):
             return True
         return False  # New uid doesn't exist
 
-    def add_widget(self, widget):
+    def add_widget(self, *args, **kwargs):
         if self.container is None:
-            super(ContentPanel, self).add_widget(widget)
+            super(ContentPanel, self).add_widget(*args, **kwargs)
         else:
-            self.container.add_widget(widget)
+            self.container.add_widget(*args, **kwargs)
 
-    def remove_widget(self, widget):
-        self.container.remove_widget(widget)
+    def remove_widget(self, *args, **kwargs):
+        self.container.remove_widget(*args, **kwargs)
 
 
 class Settings(BoxLayout):
@@ -933,6 +1001,7 @@ class Settings(BoxLayout):
         self.register_type('options', SettingOptions)
         self.register_type('title', SettingTitle)
         self.register_type('path', SettingPath)
+        self.register_type('color', SettingColor)
 
     def on_touch_down(self, touch):
         if self.collide_point(*touch.pos):
@@ -1097,11 +1166,11 @@ class InterfaceWithNoMenu(ContentPanel):
     widget.
 
     '''
-    def add_widget(self, widget):
+    def add_widget(self, *args, **kwargs):
         if self.container is not None and len(self.container.children) > 0:
             raise Exception(
                 'ContentNoMenu cannot accept more than one settings panel')
-        super(InterfaceWithNoMenu, self).add_widget(widget)
+        super(InterfaceWithNoMenu, self).add_widget(*args, **kwargs)
 
 
 class InterfaceWithTabbedPanel(FloatLayout):
@@ -1250,11 +1319,25 @@ if __name__ == '__main__':
     from kivy.app import App
 
     class SettingsApp(App):
+        demo_json_settings = json.dumps([
+            {
+                'type': 'color',
+                'title': 'Test color',
+                'desc': 'Your choosen Color',
+                'section': 'colorselection',
+                'key': 'testcolor'
+            }])
 
         def build(self):
             s = Settings()
             s.add_kivy_panel()
+            s.add_json_panel('Color settings',
+                             self.config,
+                             data=self.demo_json_settings)
             s.bind(on_close=self.stop)
             return s
 
+        def build_config(self, config):
+            config.setdefaults('colorselection', {'testcolor': '#FF0000'})
+
     SettingsApp().run()
