@@ -21,7 +21,7 @@ will automatically load the associated annotation file if it exists.
 An annotation file is JSON-based, providing a list of label dictionary items.
 The key and value must match one of the :class:`VideoPlayerAnnotation` items.
 For example, here is a short version of a jsa file that you can find in
-`examples/widgets/softboy.jsa`::
+`examples/widgets/cityCC0.jsa`::
 
 
     [
@@ -32,14 +32,14 @@ For example, here is a short version of a jsa file that you can find in
         "text": "You can change the background color"}
     ]
 
-For our softboy.avi example, the result will be:
+For our cityCC0.mpg example, the result will be:
 
 .. image:: images/videoplayer-annotation.jpg
     :align: center
 
 If you want to experiment with annotation files, test with::
 
-    python -m kivy.uix.videoplayer examples/widgets/softboy.avi
+    python -m kivy.uix.videoplayer examples/widgets/cityCC0.mpg
 
 Fullscreen
 ----------
@@ -163,9 +163,12 @@ class VideoPlayerProgressBar(ProgressBar):
         self.bubble_label = Factory.Label(text='0:00')
         self.bubble.add_widget(self.bubble_label)
         self.add_widget(self.bubble)
-        self.bind(pos=self._update_bubble,
-                  size=self._update_bubble,
-                  seek=self._update_bubble)
+
+        update = self._update_bubble
+        fbind = self.fbind
+        fbind('pos', update)
+        fbind('size', update)
+        fbind('seek', update)
 
     def on_video(self, instance, value):
         self.video.bind(position=self._update_bubble,
@@ -338,10 +341,10 @@ class VideoPlayer(GridLayout):
         video.state = 'play'
 
     :attr:`state` is an :class:`~kivy.properties.OptionProperty` and defaults
-    to 'play'.
+    to 'stop'.
     '''
 
-    play = BooleanProperty(False)
+    play = BooleanProperty(False, deprecated=True)
     '''
     .. deprecated:: 1.4.0
         Use :attr:`state` instead.
@@ -372,11 +375,11 @@ class VideoPlayer(GridLayout):
 
     '''
 
-    image_loading = StringProperty('data/images/image-loading.gif')
+    image_loading = StringProperty('data/images/image-loading.zip')
     '''Image filename used when the video is loading.
 
     :attr:`image_loading` is a :class:`~kivy.properties.StringProperty` and
-    defaults to 'data/images/image-loading.gif'.
+    defaults to 'data/images/image-loading.zip'.
     '''
 
     image_play = StringProperty(
@@ -451,7 +454,7 @@ class VideoPlayer(GridLayout):
 
     .. warning::
 
-        The re-add operation doesn't care about the index position of it's
+        The re-add operation doesn't care about the index position of its
         children within the parent.
 
     :attr:`fullscreen` is a :class:`~kivy.properties.BooleanProperty`
@@ -477,65 +480,86 @@ class VideoPlayer(GridLayout):
     # internals
     container = ObjectProperty(None)
 
+    _video_load_ev = None
+
     def __init__(self, **kwargs):
         self._video = None
         self._image = None
         self._annotations = ''
         self._annotations_labels = []
         super(VideoPlayer, self).__init__(**kwargs)
-        self._load_thumbnail()
-        self._load_annotations()
+        update_thumbnail = self._update_thumbnail
+        update_annotations = self._update_annotations
+        fbind = self.fbind
+        fbind('thumbnail', update_thumbnail)
+        fbind('annotations', update_annotations)
 
         if self.source:
             self._trigger_video_load()
 
     def _trigger_video_load(self, *largs):
-        Clock.unschedule(self._do_video_load)
-        Clock.schedule_once(self._do_video_load, -1)
+        ev = self._video_load_ev
+        if ev is None:
+            ev = self._video_load_ev = Clock.schedule_once(self._do_video_load,
+                                                           -1)
+        ev()
+
+    def _try_load_default_thumbnail(self, *largs):
+        if not self.thumbnail:
+            filename = self.source.rsplit('.', 1)
+            thumbnail = filename[0] + '.png'
+            if exists(thumbnail):
+                self._load_thumbnail(thumbnail)
+
+    def _try_load_default_annotations(self, *largs):
+        if not self.annotations:
+            filename = self.source.rsplit('.', 1)
+            annotations = filename[0] + '.jsa'
+            if exists(annotations):
+                self._load_annotations(annotations)
 
     def on_source(self, instance, value):
-        # we got a value, try to see if we have an image for it
-        self._load_thumbnail()
-        self._load_annotations()
+        # By default, VideoPlayer should look for thumbnail and annotations
+        # with the same filename (except extension) of the source (video) file.
+        Clock.schedule_once(self._try_load_default_thumbnail, -1)
+        Clock.schedule_once(self._try_load_default_annotations, -1)
         if self._video is not None:
             self._video.unload()
             self._video = None
         if value:
             self._trigger_video_load()
 
+    def _update_thumbnail(self, *largs):
+        self._load_thumbnail(self.thumbnail)
+
+    def _update_annotations(self, *largs):
+        self._load_annotations(self.annotations)
+
     def on_image_overlay_play(self, instance, value):
         self._image.image_overlay_play = value
 
     def on_image_loading(self, instance, value):
         self._image.image_loading = value
 
-    def _load_thumbnail(self):
+    def _load_thumbnail(self, thumbnail):
         if not self.container:
             return
         self.container.clear_widgets()
-        # get the source, remove extension, and use png
-        thumbnail = self.thumbnail
-        if not thumbnail:
-            filename = self.source.rsplit('.', 1)
-            thumbnail = filename[0] + '.png'
-        self._image = VideoPlayerPreview(source=thumbnail, video=self)
-        self.container.add_widget(self._image)
+        if thumbnail:
+            self._image = VideoPlayerPreview(source=thumbnail, video=self)
+            self.container.add_widget(self._image)
 
-    def _load_annotations(self):
+    def _load_annotations(self, annotations):
         if not self.container:
             return
         self._annotations_labels = []
-        annotations = self.annotations
-        if not annotations:
-            filename = self.source.rsplit('.', 1)
-            annotations = filename[0] + '.jsa'
-        if exists(annotations):
+        if annotations:
             with open(annotations, 'r') as fd:
                 self._annotations = load(fd)
-        if self._annotations:
-            for ann in self._annotations:
-                self._annotations_labels.append(
-                    VideoPlayerAnnotation(annotation=ann))
+            if self._annotations:
+                for ann in self._annotations:
+                    self._annotations_labels.append(
+                        VideoPlayerAnnotation(annotation=ann))
 
     def on_state(self, instance, value):
         if self._video is not None:
@@ -576,17 +600,29 @@ class VideoPlayer(GridLayout):
             elif label.parent is None:
                 self.container.add_widget(label)
 
-    def seek(self, percent):
-        '''Change the position to a percentage of the duration. Percentage must
-        be a value between 0-1.
+    def seek(self, percent, precise=True):
+        '''Change the position to a percentage (strictly, a proportion)
+           of duration.
+
+        :Parameters:
+            `percent`: float or int
+                Position to seek as a proportion of total duration, must
+                be between 0-1.
+            `precise`: bool, defaults to True
+                Precise seeking is slower, but seeks to exact requested
+                percent.
 
         .. warning::
+            Calling seek() before the video is loaded has no effect.
 
-            Calling seek() before video is loaded has no effect.
+        .. versionadded:: 1.2.0
+
+        .. versionchanged:: 1.10.1
+            The `precise` keyword argument has been added.
         '''
         if not self._video:
             return
-        self._video.seek(percent)
+        self._video.seek(percent, precise=precise)
 
     def _play_started(self, instance, value):
         self.container.clear_widgets()
@@ -634,7 +670,7 @@ class VideoPlayer(GridLayout):
             window.add_widget(self)
 
             # ensure the video widget is in 0, 0, and the size will be
-            # reajusted
+            # readjusted
             self.pos = (0, 0)
             self.size = (100, 100)
             self.pos_hint = {}
